// InitWnd.cpp : implementation file
//

#include "stdafx.h"
#include "msp.h"
#include "InitWnd.h"
#include "Gos.h"
#include "MCI.h"
#include <mmsystem.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDC_ERROR		100
#define IDC_STATUS		101
#define IDC_INPUT		102
#define IDC_QUEUE		103

/////////////////////////////////////////////////////////////////////////////
// CInitWnd

CInitWnd::CInitWnd()
{
	m_CreateWin();
}

CInitWnd::CInitWnd(int port, HWND hOwner)
{
	m_port = port;
	m_CreateWin(hOwner);
}

void CInitWnd::m_CreateWin(HWND hOwner)
{
	CString sClass = ::AfxRegisterWndClass(
		0,
		AfxGetApp()->LoadStandardCursor(IDC_ARROW),
		(HBRUSH)(COLOR_3DFACE +1),
		AfxGetApp()->LoadIcon(IDI_GOSICON)
		);

	CreateEx(0, sClass, _T("Gos Plug-In"), WS_OVERLAPPEDWINDOW,
		0, 731, 1280, 265, NULL, NULL);

	m_wave.nPriority = -1; /*imposta la priorita' del suono corrente a -1
	in modo che qualsiasi altro suono abbia una priorita' piu' alta*/
}

CInitWnd::~CInitWnd()
{
}


BEGIN_MESSAGE_MAP(CInitWnd, CWnd)
	//{{AFX_MSG_MAP(CInitWnd)
	ON_WM_CREATE()
	ON_MESSAGE(GS_INPUT, OnGosInput)
	ON_MESSAGE(GS_OUTPUT, OnGosOutput)
	ON_MESSAGE(GS_NOTIFY, OnGosNotify)
	ON_MESSAGE(GS_GO, OnGosGo)
	ON_MESSAGE(GS_CLOSE, OnGosClose)
	ON_MESSAGE(MM_MCINOTIFY, OnMciNotify)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CInitWnd message handlers

void CInitWnd::PostNcDestroy() 
{
	// TODO: Add your specialized code here and/or call the base class
	m_gos->gosClose();
	delete m_gos;
	delete this;
	
	CWnd::PostNcDestroy();
}

int CInitWnd::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	m_gos = new CGosCommand(this->m_hWnd);
	m_gos->Create();
	m_gos->gosConnect(m_port);
	return 0;
}

LRESULT CInitWnd::OnGosInput(WPARAM wParam, LPARAM lParam)
{
	return 0;
}

LRESULT CInitWnd::OnGosOutput(WPARAM wParam, LPARAM lParam)
{
	gosOutputStruct* pgos = (gosOutputStruct*)lParam;
	
	if (pgos->nType != TOUT_FIRST)
		return 0;

	//m_gos->gosSendOutput("-> ricezione output iniziale");

	CString sData = pgos->lpOut;
	//sData.MakeLower();
	
	int iniPos, finPos;
	iniPos = sData.Find("!!MUSIC");
	if (iniPos > -1)
	{
		//m_gos->gosSendOutput("-> trovato !!MUSIC");		
		finPos = sData.Find(")", iniPos + 1);
		CString sFinal = sData.Left(iniPos);
		sFinal += sData.Mid(finPos + 1);

		ParseMUSICDirective(sData, iniPos);
		//m_gos->gosSendOutput("-> processata musica");
		m_gos->gosStopOutput(sFinal);
		//m_gos->gosSendOutput("-> inviato messaggio di stop");
		return 0;
	}

	iniPos = sData.Find("!!SOUND");
	if (iniPos > -1)
	{
		//m_gos->gosSendOutput("-> trovato !!MUSIC");		
		finPos = sData.Find(")", iniPos + 1);
		CString sFinal = sData.Left(iniPos);
		sFinal += sData.Mid(finPos + 1);

		ParseSOUNDDirective(sData, iniPos);
		//m_gos->gosSendOutput("-> processata musica");
		m_gos->gosStopOutput(sFinal);
		//m_gos->gosSendOutput("-> inviato messaggio di stop");
		return 0;

	}

	m_gos->gosLeaveOutput();
	//m_gos->gosSendOutput("-> inviato messaggio di non-stop");
	return 0;
}

LRESULT CInitWnd::OnGosNotify(WPARAM wParam, LPARAM lParam)
{
	switch (wParam)
	{
		case ENVM_CLOSE: //la connessione e' chiusa
			m_mci.CloseAll();
			break;
		default:
			return 0;
	}

	return 0;
}

LRESULT CInitWnd::OnGosGo(WPARAM wParam, LPARAM lParam)
{
	m_gos->gosSendOutput(_T("Mud Sound Protocol Plug-in 1.1"));
	m_gos->gosSendOutput(_T("Sviluppato da Seph (gosclient@yahoo.it)"));
	return 0;
}

LRESULT CInitWnd::OnGosClose(WPARAM wParam, LPARAM lParam)
{
	m_gos->gosClose();
	DestroyWindow();
	return 0;
}

LRESULT CInitWnd::OnMciNotify(WPARAM wParam, LPARAM lParam)
{
	WORD wFlags = wParam;
	long lDevID = lParam;


	if(wFlags == MCI_NOTIFY_SUCCESSFUL) {
		int nDevType = m_mci.GetDeviceType(lDevID);
		
		if (nDevType == MCI_WAVEID) 
		{
			BOOL bLoop = FALSE;
			if (m_wave.nRepeats == -1) bLoop = TRUE;
			if (m_wave.nRepeats > 0) { m_wave.nRepeats--; bLoop = TRUE; }
			if (bLoop) m_mci.RestartWave(m_hWnd);
				else
				{
					m_mci.CloseDevice(lDevID);
					m_wave.nPriority = -1;
				}
		}

		if (nDevType == MCI_MIDIID)
		{
			
			BOOL bLoop = FALSE;
			if (m_midi.nRepeats == -1) bLoop = TRUE;
			if (m_midi.nRepeats > 0) { m_midi.nRepeats--; bLoop = TRUE; }
			if (bLoop) m_mci.RestartMidi(m_hWnd);
				else
				{
					m_mci.CloseDevice(lDevID);
					m_midi.sFName = "";
				}
		}
	}

	return 0;
}

void CInitWnd::ParseMUSICDirective(CString sData, int pos)
{
	MSPFile msp_midi;
	BOOL ret = ParseMSPDirective(msp_midi, sData, pos);

	if (ret == TRUE)
	{
		msp_midi.sFName = m_gos->gosMudPath() + msp_midi.sFName;
		BOOL bPlay = TRUE;
		if (msp_midi.sFName.CompareNoCase(m_midi.sFName) == 0)
		{
			//e' stato richiesto di suonare lo stesso file
			bPlay = FALSE;
			if (m_midi.nContinue == 0)
				m_mci.RestartMidi(m_hWnd);
		}
		m_midi = msp_midi;
		m_gos->gosSendOutput(msp_midi.sFName);
		
		m_mci.SetMidiVol(m_midi.nVol);
		if (bPlay == TRUE)
		{
			DWORD rtn = m_mci.playMIDIFile(m_hWnd, m_midi.sFName);
			if (rtn != 0L) //an error occured
			{
				ShowMCIError(rtn);
				m_midi.sFName = "";
			}
		}
	}
}

void CInitWnd::ParseSOUNDDirective(CString sData, int pos)
{
	MSPFile msp_wave;
	BOOL ret = ParseMSPDirective(msp_wave, sData, pos);

	if (ret == TRUE)
	{
		msp_wave.sFName = m_gos->gosMudPath() + msp_wave.sFName;
		if (m_wave.nPriority >= msp_wave.nPriority)
			return;

		m_wave = msp_wave;
		m_gos->gosSendOutput(msp_wave.sFName);
		
		m_mci.SetWaveVol(m_wave.nVol);
		DWORD rtn = m_mci.playWAVEFile(m_hWnd, msp_wave.sFName);
		if (rtn != 0L) //an error occured
		{
			ShowMCIError(rtn);
			m_wave.nPriority = -1;
		}
	}
}

BOOL CInitWnd::ParseMSPDirective(MSPFile &dest, CString sData, int pos)
{
	//inserisce nella struttura i valori di default
		dest.nVol = 100;
		dest.nRepeats = 0;
		dest.nPriority = 50;
		dest.nContinue = 1;
	//~

	int nDevType;
	if (sData.Mid(pos, 7) == "!!MUSIC") nDevType = MCI_MIDIID;
	if (sData.Mid(pos, 7) == "!!SOUND") nDevType = MCI_WAVEID;

	sData = sData.Mid(pos + 8);
	sData.Replace(")", " ");
	if (sData.Left(3) == "Off")
	{
		if (nDevType == MCI_MIDIID){ m_mci.CloseMidi(); m_midi.sFName = ""; }
		if (nDevType == MCI_WAVEID){ m_mci.CloseWave(); m_wave.nPriority = -1; }
		return FALSE;
	}

	pos = sData.Find(" ");
	dest.sFName = sData.Left(pos + 1);
	//CString sTitle = sData.Left(pos + 1);
	dest.sFName.Replace("/", "\\");
	//sTitle.Replace("/", "\\");
	//m_Log(sTitle);

	int finPos;
	pos = sData.Find("V=");
	if (pos > -1) //estrazione del volume
	{
		finPos = sData.Find(" ", pos + 1);
		dest.nVol = _ttoi(sData.Mid(pos + 2, finPos - (pos + 2)));
	}

	pos = sData.Find("L=");
	if (pos > -1) //estrazione delle ripetizioni
	{
		finPos = sData.Find(" ", pos + 1);
		dest.nRepeats = _ttoi(sData.Mid(pos + 2, finPos - (pos + 2)));
		if (dest.nRepeats > 0) dest.nRepeats--;
	}

	pos = sData.Find("C=");
	if (pos > -1) //estrazione del "continue"
	{
		finPos = sData.Find(" ", pos + 1);
		dest.nContinue = _ttoi(sData.Mid(pos + 2, finPos - (pos + 2)));
	}

	pos = sData.Find("P=");
	if (pos > -1) //estrazione della priorita'
	{
		finPos = sData.Find(" ", pos + 1);
		dest.nPriority = _ttoi(sData.Mid(pos + 2, finPos - (pos + 2)));
	}

	pos = sData.Find("T=");
	if (pos > -1) //estrazione del tipo
	{
		finPos = sData.Find(" ", pos + 1);
		dest.sType = sData.Mid(pos + 2, finPos - (pos + 2));
	}

	pos = sData.Find("U=");
	if (pos > -1) //estrazione dell'url per eventuale download
	{
		finPos = sData.Find(" ", pos + 1);
		dest.sURL = sData.Mid(pos + 2, finPos - (pos + 2));
	}

	return TRUE;
}

void CInitWnd::ShowMCIError(DWORD err)
{
	char cBuff[128];
	mciGetErrorString((MCIERROR) err, cBuff, 128);
	CString sBuff = cBuff;
	sBuff.TrimRight();
	m_gos->gosSendOutput(sBuff, TOUT_ERROR);
}
