// MCI.cpp: implementation of the CMCI class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <mmsystem.h>		//dichiarazioni per l'MCI
#include "AlexfMixer.h"
#include "msp.h"
#include "MCI.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMCI::CMCI()
{
	m_midiID = 0;
	m_waveID = 0;
	m_midiMx = new CAlexfMixer(MIXERLINE_COMPONENTTYPE_SRC_SYNTHESIZER,
		NO_SOURCE, MIXERCONTROL_CONTROLTYPE_VOLUME);
	m_midiIniVol = m_midiMx->GetControlValue();
	m_waveMx = new CAlexfMixer(MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT,
		NO_SOURCE, MIXERCONTROL_CONTROLTYPE_VOLUME);
	m_waveIniVol = m_waveMx->GetControlValue();
}

CMCI::~CMCI()
{
	m_midiMx->SetControlValue(m_midiIniVol);
	delete m_midiMx;
	m_waveMx->SetControlValue(m_waveIniVol);
	delete m_waveMx;
	CloseAll();
}

DWORD CMCI::playWAVEFile(HWND hWndNotify, LPCTSTR lpszWAVEFileName)
{
    UINT wDeviceID;
    DWORD dwReturn;
    MCI_OPEN_PARMS mciOpenParms;
    MCI_PLAY_PARMS mciPlayParms;

	CloseDevice(m_waveID);

    // Open the device by specifying the device and filename.
    // MCI will choose a device capable of playing the specified file.

    mciOpenParms.lpstrDeviceType = "waveaudio";
    mciOpenParms.lpstrElementName = lpszWAVEFileName;
    if (dwReturn = mciSendCommand(0, MCI_OPEN,
       MCI_OPEN_TYPE | MCI_OPEN_ELEMENT, 
       (DWORD)(LPVOID) &mciOpenParms))
    {
        // Failed to open device. Don't close it; just return error.
        return (dwReturn);
    }

    // The device opened successfully; get the device ID.
    wDeviceID = mciOpenParms.wDeviceID;

    // Begin playback. The window procedure function for the parent 
    // window will be notified with an MM_MCINOTIFY message when 
    // playback is complete. At this time, the window procedure closes 
    // the device.

    mciPlayParms.dwCallback = (DWORD) hWndNotify;
    if (dwReturn = mciSendCommand(wDeviceID, MCI_PLAY, MCI_NOTIFY, 
        (DWORD)(LPVOID) &mciPlayParms))
    {
        mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL);
        return (dwReturn);
    }
	
	m_waveID = wDeviceID;
    return (0L);
}

DWORD CMCI::playMIDIFile(HWND hWndNotify, LPCTSTR lpszMIDIFileName)
{
    UINT wDeviceID;
    DWORD dwReturn;
    MCI_OPEN_PARMS mciOpenParms;
    MCI_PLAY_PARMS mciPlayParms;
    MCI_STATUS_PARMS mciStatusParms;
//    MCI_SEQ_SET_PARMS mciSeqSetParms;

	CloseDevice(m_midiID);
    // Open the device by specifying the device and filename.
    // MCI will attempt to choose the MIDI mapper as the output port.
    mciOpenParms.lpstrDeviceType = "sequencer";
    mciOpenParms.lpstrElementName = lpszMIDIFileName;
    if (dwReturn = mciSendCommand(NULL, MCI_OPEN,
        MCI_OPEN_TYPE | MCI_OPEN_ELEMENT,
        (DWORD)(LPVOID) &mciOpenParms))
    {
        // Failed to open device. Don't close it; just return error.
        return (dwReturn);
    }

    // The device opened successfully; get the device ID.
    wDeviceID = mciOpenParms.wDeviceID;

    // Check if the output port is the MIDI mapper.
    mciStatusParms.dwItem = MCI_SEQ_STATUS_PORT;
    if (dwReturn = mciSendCommand(wDeviceID, MCI_STATUS, 
        MCI_STATUS_ITEM, (DWORD)(LPVOID) &mciStatusParms))
    {
        mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL);
        return (dwReturn);
    }

    // The output port is not the MIDI mapper. 
    // Ask if the user wants to continue.
    /*if (LOWORD(mciStatusParms.dwReturn) != MIDI_MAPPER)
    {
        if (MessageBox(hWndNotify,
            "The MIDI mapper is not available. Continue?",
            "", MB_YESNO) == IDNO)
        {
            // User does not want to continue. Not an error;
            // just close the device and return.
            mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL);
            return (0L);
        }
    }*/

    // Begin playback. The window procedure function for the parent 
    // window will be notified with an MM_MCINOTIFY message when 
    // playback is complete. At this time, the window procedure closes 
    // the device.
    mciPlayParms.dwCallback = (DWORD) hWndNotify;
    if (dwReturn = mciSendCommand(wDeviceID, MCI_PLAY, MCI_NOTIFY, 
        (DWORD)(LPVOID) &mciPlayParms))
    {
        mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL);
        return (dwReturn);
    }

	m_midiID = wDeviceID;
    return (0L);
}

int CMCI::CloseDevice(UINT id)
{
	int IDType;
	if (!(id == 0))
	{
		if (id == m_midiID) { IDType = MCI_MIDIID; m_midiID = 0;}
		if (id == m_waveID) { IDType = MCI_WAVEID; m_waveID = 0;}
		mciSendCommand(id, MCI_STOP, MCI_WAIT, NULL);
		mciSendCommand(id, MCI_CLOSE, MCI_WAIT, NULL);
	};
	return IDType;
}

void CMCI::CloseAll()
{
	CloseDevice(m_midiID);
	CloseDevice(m_waveID);
}

void CMCI::CloseMidi()
{
	CloseDevice(m_midiID);
}

void CMCI::CloseWave()
{
	CloseDevice(m_waveID);
}

void CMCI::RestartMidi(HWND hWndNotify)
{
	if (!(m_midiID == 0))
	{
		mciSendCommand(m_midiID, MCI_SEEK, MCI_WAIT | MCI_SEEK_TO_START, NULL);
		MCI_PLAY_PARMS mciPlayParms;
		mciPlayParms.dwCallback = (DWORD) hWndNotify;
		mciSendCommand(m_midiID, MCI_PLAY, MCI_NOTIFY, 
			(DWORD)(LPVOID) &mciPlayParms);
	}
}

void CMCI::RestartWave(HWND hWndNotify)
{
	if (!(m_waveID == 0))
	{
		mciSendCommand(m_waveID, MCI_SEEK, MCI_WAIT | MCI_SEEK_TO_START, NULL);
		MCI_PLAY_PARMS mciPlayParms;
		mciPlayParms.dwCallback = (DWORD) hWndNotify;
		mciSendCommand(m_waveID, MCI_PLAY, MCI_NOTIFY, 
			(DWORD)(LPVOID) &mciPlayParms);
	}
}

int CMCI::GetDeviceType(UINT id)
{
	if (id == m_midiID) return MCI_MIDIID;
	if (id == m_waveID) return MCI_WAVEID;
	return 0;
}

void CMCI::SetMidiVol(int vol)
{
	DWORD curVol = m_midiMx->GetControlValue();
	if (!((DWORD)vol == curVol))
		m_midiMx->SetControlValue(vol);
}

void CMCI::SetWaveVol(int vol)
{
	DWORD curVol = m_waveMx->GetControlValue();
	if (!((DWORD)vol == curVol))
		m_waveMx->SetControlValue(vol);
}
